"""
   Implement low-level objects for popen() for DOS

   Initially written by BroytMann. Jan 1997.
"""


import os
from FileWrapper import FileWrapper
import tempfile


class DosPipe(FileWrapper):
   """
      Abstract object that inherits FileWrapper and provides base
      for read/write pipeobjects
   """
   def __init__(self):
      """
         Generates temporary filename and creates temporary file
         for reading/writing
      """
      self.tempname = tempfile.mktemp()
      self.f = open(self.tempname, 'w+')
      FileWrapper.__init__(self, self.f)

   def close(self):
      """
         Closes and removes temporary file
      """
      FileWrapper.close(self) # This only flushes the file
      self.f.close()
      os.unlink(self.tempname)


class ReadPipe(DosPipe):
   """
      Object that provides read interface for popen() for DOS.
      The object acts as fileobject (and it really inherits methods
      of fileobject via FileWrapper). Only reading allowed.
   """
   def __init__(self, cmd):
      """
         With DOS, all results of command must be available before
         reading the pipe. So the command run here, just when pipe
         is opening for reading. Results of command are collected in
         our temporary file.
      """
      DosPipe.__init__(self)
      os.system(cmd + " > " + self.tempname)

   def write(self, *args):
      raise IOError, "attempt to write to read-only pipe"

   writelines = write

class WritePipe(DosPipe):
   """
      Object that provides write interface for popen() for DOS.
      The object acts as fileobject (and it really inherits methods
      of fileobject via FileWrapper). Only writing allowed.
   """
   def __init__(self, cmd):
      """
         Open "pipe" for writing. Here we only save the command
         for later run.
      """
      DosPipe.__init__(self)
      self.cmd = cmd

   def close(self):
      """
         With DOS, all data for command must be available after
         all writing to the pipe. So the command run just when pipe
         is closing. Results for command are collected in
         our temporary file during writing to the pipe.
      """
      self.f.flush() # Flush all data for our file to disk
      os.system(self.cmd + " < " + self.tempname) # Run the command...
      DosPipe.close(self) #... and immediately close the pipe

   def read(self, *args):
      raise IOError, "attempt to read from write-only pipe"

   readline  = read
   readlines = read
